
/*********************
  Balancing of cells
*********************/
boolean balanceCells(uint16_t lowestCellVoltage)
{
  static uint8_t counter = 1;
  static uint16_t cellToBalanceFilter = 0x0FFF & 0x5555; // Every odd cell
  boolean measureVoltages = false;

  if(1 == counter) Serial.println("Balance cells");

  writeAllSlaves(0x18, 0x1500, true); // Set WATCHDOG timer to 5s

  if (0 == counter)
  {
    cellToBalanceFilter = 0x0FFF & ~cellToBalanceFilter; // Every other cell
    writeAllSlaves(0x1A, 0x0000, true); // Turn off all cell balancing for one cycle to avoid any switch overlap
  }

  if (1 == counter)
  {
    measureVoltages = true; // OK to measure cell voltages
  }

  for (int i = 0; i < NumModulesFound; i++) // Go through modules to set cells for balancing
  {
    if (2 == counter) // Only set cells once every counter cycle
    {
      uint16_t cellToBalance = 0x0000;

      for (int j = 0; j < MAX_CELLS_PER_MODULE; j++) // Go through cells to set cells for balancing
      {
        uint8_t cellNumber = j + i * MAX_CELLS_PER_MODULE;
        Serial.print(j);
        Serial.print(" ");
        Serial.print(cellVoltageNew[i][j]);
        Serial.print(" ");
        // Serial.println(startShuntVoltage);
        //  Serial.println(lowestCellVoltage);
        if ((cellVoltageNew[i][j] > startShuntVoltage) && ((int16_t)(cellVoltageNew[i][j] - lowestCellVoltage) > voltageAllowance)) // Ok to balance
        {
          cellToBalance |= (0x01 << j);

        }
        /*     else //Error
             {
               cellToBalance = 0x0000;
             }*/

      }
      Serial.println("");
      Serial.print("Module: ");
      Serial.println(i);
      Serial.println("All Cells to Balance");
      Serial.println(cellToBalance, BIN);
      cellToBalance &= cellToBalanceFilter; // To avoid enabling adjacent balance switches
      writeAddressedSlave(0x1A, cellToBalance, i, true); // Send request to set cell balance switches
      Serial.print("Cell to Balance Filtered :");
        Serial.println(cellToBalance, BIN);

      for (int bi = 0; bi < MAX_CELLS_PER_MODULE; bi++)
      {
        if ( (cellToBalance & 0x01 << bi) == 0x01 << bi)
        {
          Serial.print(bi);
          Serial.print(" ");
        }
      }
      Serial.print(" Voltage Diff Min-Max: ");
      Serial.println(CellData.MaxVoltage.Voltage - CellData.MinVoltage.Voltage);

    }

    if (dieTemperature[i] > 100) // Check die overtemp
    {
      writeAddressedSlave(0x1A, 0x0000, i, true); // Turn off cell balancing for the module with die overtemp
      Serial.print("Die overtemp, module ");
      Serial.println(i + 1);
    }
  }

  counter++;

  if (counter > balanceCellCounterTurnAround)
  {
    counter = 0;
  }

  return measureVoltages;
}
