
/*****************************************
  Reads data from addressed slave device
*****************************************/
void readAddressedSlave(uint8_t dataRegister, uint8_t address, boolean setupDone) // Read addressed slave
{
  uint8_t command = 0x05; // READDEVICE
  command |= (address << 3);
  uint8_t byteList[3] = {command, dataRegister, 0x00};
  uint8_t PEC = calculatePEC(byteList, 3);
  uint8_t readRegisterData[7];
  uint8_t errorByte = 0x00;
  static uint8_t resendCounter = 0;
  static uint8_t failCounter = 0;

  if (failCounter >= 4)
  {
    clearBuffers();
    failCounter = 0; // Reset counter
    if (DebugStatus & DEBUG_RX)
      Serial.println("READ Fail Buffers cleared");
  }

  SPI.beginTransaction(MAX17841);

  // 1, Load the READDEVICE command sequence into the load queue
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0xC0); // WR_LD_Q SPI command byte (write the load queue)
  SPI.transfer(0x1d); // Message length (5 + 2 x n = 29)
  SPI.transfer(command); // READALL command byte
  SPI.transfer(dataRegister); // Register address
  SPI.transfer(0x00); // Data-check byte (seed value = 0x00)
  SPI.transfer(PEC); // PEC byte
  SPI.transfer(0x00); // Alive-counter byte (seed value = 0x00)
  digitalWrite(SPI_MAX_CS_PIN, HIGH);

  transmitQueue();

  // 4, Read the receive buffer
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0x93); // RD_NXT_MSG SPI command byte

  for (int i = 0; i < 7; i++)
  {
    readRegisterData[i] = SPI.transfer(0x93);
  }

  errorByte |= receiveBufferError();

  SPI.endTransaction();

  // Verify that the device register data is received correctly during the READDEVICE sequence
  if (!((readRegisterData[0] == command) && (readRegisterData[1] == dataRegister)))
  {
    errorByte |= mismatchBefore;
  }
  if (setupDone)
  {
    uint8_t checkPEC = calculatePEC(readRegisterData, 5);
    // Check check-byte, PEC and alive-counter
    if (!((readRegisterData[4] == 0x00) && (readRegisterData[5] == checkPEC) && (readRegisterData[6] == 0x01)))
    {
      errorByte |= mismatchAfter;
    }
  }

  // Print data received
  if (DebugStatus & DEBUG_RX)
  {
    for (int i = 0; i < 7; i++)
    {
      Serial.print(readRegisterData[i], HEX);
      Serial.print(" ");
    }
    Serial.println();
  }

  if (errorByte) // Error
  {
    resendCounter++;
    Serial.println(errorByte, HEX);
    errorByte &= 0x00; // Clear errors
    Serial.println("errorByte cleared");

    if (resendCounter > 2)
    {
      if (DebugStatus & DEBUG_RX)
        Serial.println("READEVICE fail");
      resendCounter = 0;
      failCounter++;
      return;
    }

    delay(1);
    readAddressedSlave(dataRegister, address, setupDone); // Resend READDEVICE
  }
  else // No errors, clear counters and store data received
  {
    resendCounter = 0; // Reset counter
    failCounter = 0; // Reset counter
  }
}
