
/************************************
  Reads data from all slave devices
************************************/
void readAllSlaves(uint8_t dataRegister, boolean setupDone) // Read all slaves
{
  uint8_t command = 0x03; // READALL
  uint8_t byteList[3] = {command, dataRegister, 0x00};
  uint8_t PEC = calculatePEC(byteList, 3);
  uint8_t readRegisterData[49];
  uint8_t errorByte = 0x00;
  uint8_t packet_len = 0x00;
  static uint8_t resendCounter = 0;
  static uint8_t failCounter = 0;

  if (failCounter >= 4)
  {
    clearBuffers();
    failCounter = 0; // Reset counter
    if (DebugStatus & DEBUG_ERRORS)
      Serial.println("Buffers cleared");
  }

  SPI.beginTransaction(MAX17841);

  packet_len = 5 + (2 * NumModulesFound);

  // Load the READALL command sequence into the load queue
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0xC0); // WR_LD_Q SPI command byte (write the load queue)
  SPI.transfer(packet_len); // Message length (5 + 2 x n = 29)
  SPI.transfer(command); // READALL command byte
  SPI.transfer(dataRegister); // Register address
  SPI.transfer(0x00); // Data-check byte (seed value = 0x00)
  SPI.transfer(PEC); // PEC byte
  SPI.transfer(0x00); // Alive-counter byte (seed value = 0x00)
  digitalWrite(SPI_MAX_CS_PIN, HIGH);

  transmitQueue();

  // Read the receive buffer
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0x93); // RD_NXT_MSG SPI command byte

  for (int i = 0; i < packet_len; i++)
  {
    readRegisterData[i] = SPI.transfer(0x93);
  }

  /*  for (int i = 0; i < packet_len; i++)
    { Serial.print(readRegisterData[i], HEX);
     Serial.print(" ");
    }
    Serial.println(" ");
  */
  errorByte |= receiveBufferError();

  SPI.endTransaction();

  // Verify that the device register data is received correctly during the READALL sequence
  if (!((readRegisterData[0] == command) && (readRegisterData[1] == dataRegister)))
  {
    errorByte |= mismatchBefore;
  }
  if (setupDone)
  {
    uint8_t PECbyte = 4 + (2 * NumModulesFound);
    uint8_t checkPEC = calculatePEC(readRegisterData, PECbyte - 1); // Changed to 7 to read 2 modules
    // Check check-byte, PEC and alive-counter
    if (!((readRegisterData[PECbyte - 2] == 0x00) && (readRegisterData[PECbyte - 1] == checkPEC) && (readRegisterData[PECbyte] == NumModulesFound))) // Changed to 6, 7, 8 which works for two slaves(why?). no values above [8] when 2 modules are connected. 26, 27, 28 when using 12 slaves.
    {
      errorByte |= mismatchAfter;
      if (DebugStatus & DEBUG_RX)
      {
        Serial.print("PEC:");
        Serial.print(checkPEC, HEX);
      }
    }
  }

  // Print data received
  if (DebugStatus & DEBUG_RX)
  {
    for (int i = 0; i < packet_len; i++)
    {
      Serial.print("R:");
      Serial.print(readRegisterData[i], HEX);
      Serial.print(" ");
    }
    Serial.println();
  }
  if (errorByte) // Error
  {
    resendCounter++;
    errorByte &= 0x00; // Clear errors
    if (DebugStatus & DEBUG_RX)
    {
      Serial.println(errorByte, HEX);
      Serial.println("errorByte cleared");
    }

    if (resendCounter > 2)
    {
      if (DebugStatus & DEBUG_RX)
        Serial.println("READALL fail");
      resendCounter = 0;
      failCounter++;
      return;
    }

    delay(1);
    //   readAllSlaves(dataRegister, setupDone); // Resend READALL
  }
  else // No errors, clear counters and store data received
  {
    resendCounter = 0; // Reset counter
    failCounter = 0; // Reset counter

    if ((dataRegister >= 0x20) && (dataRegister <= 0x2B)) // Cell voltage measurements
    {
      storeCellVoltage(dataRegister, readRegisterData);
    }

    if (dataRegister == 0x2D) // Aux voltage measurements (external temperature sensors)
    {
      storeCellTemperature(dataRegister, readRegisterData);
    }
    if (dataRegister == 0x2E) // Aux voltage measurements (external temperature sensors)
    {
      storeCellTemperature(dataRegister, readRegisterData);
    }

    if (dataRegister == 0x50) // Die temperature measurements
    {
      storeDieTemperature(readRegisterData);
    }

    if (dataRegister == 0x13) // Read SCANCTRL to check if data is ready to be read
    {
      checkDataReady(readRegisterData);
    }
  }
}
