
/****************************************
  Writes data to addressed slave device
****************************************/
void writeAddressedSlave(uint8_t dataRegister, uint16_t data, uint8_t address, boolean setupDone) // Write addressed slave
{
  uint8_t command = 0x04; // WRITEDEVICE
  command |= (address << 3);
  uint8_t byteList[4] = {command, dataRegister, lowByte(data), highByte(data)};
  uint8_t PEC = calculatePEC(byteList, 4);
  uint8_t readRegisterData[6];
  uint8_t errorByte = 0x00;
  static uint8_t resendCounter = 0;
  static uint8_t failCounter = 0;

  if (failCounter >= 4)
  {
    clearBuffers();
    failCounter = 0; // Reset counter
    Serial.println("Buffers cleared");
  }

  SPI.beginTransaction(MAX17841);

  // 1, Load the WRITEDEVICE command sequence into the load queue
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0xC0); // WR_LD_Q SPI command byte (write the load queue)
  SPI.transfer(0x06); // Message length
  SPI.transfer(command); // WRITEDEVICE command byte
  SPI.transfer(dataRegister); // Register address
  SPI.transfer(lowByte(data)); // LS byte of register data to be written
  SPI.transfer(highByte(data)); // MS byte of register data to be written
  SPI.transfer(PEC); // PEC byte
  SPI.transfer(0x00); // Alive-counter byte (seed value = 0x00)
  digitalWrite(SPI_MAX_CS_PIN, HIGH);

  transmitQueue();

  // 4, Read the receive buffer
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0x93); // RD_NXT_MSG SPI command byte

  for (int i = 0; i < 6; i++)
  {
    readRegisterData[i] = SPI.transfer(0x93);
  }
  digitalWrite(SPI_MAX_CS_PIN, HIGH);

  errorByte |= receiveBufferError();

  SPI.endTransaction();

  // Verify that the device register data is what was written during the WRITEDEVICE sequence
  if (!((readRegisterData[0] == command) && (readRegisterData[1] == dataRegister) && (readRegisterData[2] == lowByte(data)) && (readRegisterData[3] == highByte(data)) && (readRegisterData[4] == PEC)))
  {
    errorByte |= mismatchBefore;
  }
  if (setupDone)
  {
    // Check alive-counter
    if (!(readRegisterData[5] == 0x01)) //Changed from All Modules to 1
    {
      errorByte |= mismatchAfter;
    }
  }

  /*// Print data received
    for(int i=0; i<6; i++)
    {
    Serial.print(readRegisterData[i], HEX);
    Serial.print(" ");
    }
    Serial.println();*/

  if (errorByte) // Error
  {
    resendCounter++;
    Serial.println(errorByte, HEX);
    errorByte &= 0x00; // Clear errors
    Serial.println("errorByte cleared");

    if (resendCounter > 2)
    {
      Serial.println("WRITEDEVICE fail");
      resendCounter = 0;
      failCounter++;
      return;
    }

    delay(1);
    writeAddressedSlave(dataRegister, data, address, setupDone); // Resend WRITEDEVICE
  }
  else // No errors, clear counters
  {
    resendCounter = 0; // Reset counter
    failCounter = 0; // Reset counter
  }
}
