
/***********************************
  Writes data to all slave devices
***********************************/
void writeAllSlaves(uint8_t dataRegister, uint16_t data, boolean setupDone)
{
  uint8_t command = 0x02; // WRITEALL
  uint8_t byteList[4] = {command, dataRegister, lowByte(data), highByte(data)};
  uint8_t PEC = calculatePEC(byteList, 4);
  uint8_t readRegisterData[6];
  uint8_t errorByte = 0x00;
  static uint8_t resendCounter = 0;
  static uint8_t failCounter = 0;

  if (failCounter >= 4)
  {
    clearBuffers();
    failCounter = 0; // Reset counter
    Serial.println("Buffers cleared");
  }

  SPI.beginTransaction(MAX17841);

  // 1, Load the WRITEALL command sequence into the load queue
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0xC0); // WR_LD_Q SPI command byte (write the load queue)
  SPI.transfer(0x06); // Message length
  SPI.transfer(command); // WRITEALL command byte
  SPI.transfer(dataRegister); // Register address
  SPI.transfer(lowByte(data)); // LS byte of register data to be written
  SPI.transfer(highByte(data)); // MS byte of register data to be written
  SPI.transfer(PEC); // PEC byte
  SPI.transfer(0x00); // Alive-counter byte (seed value = 0x00)
  digitalWrite(SPI_MAX_CS_PIN, HIGH);

  transmitQueue();

  // 4, Read the receive buffer
  digitalWrite(SPI_MAX_CS_PIN, LOW);
  SPI.transfer(0x93); // RD_NXT_MSG SPI command byte

  for (int i = 0; i < 6; i++)
  {
    readRegisterData[i] = SPI.transfer(0x93);
  }
  digitalWrite(SPI_MAX_CS_PIN, HIGH);

  errorByte |= receiveBufferError();

  SPI.endTransaction();

  // Verify that the device register data is what was written during the WRITEALL sequence
  if (!((readRegisterData[0] == command) && (readRegisterData[1] == dataRegister) && (readRegisterData[2] == lowByte(data)) && (readRegisterData[3] == highByte(data)) && (readRegisterData[4] == PEC)))
  {
    errorByte |= mismatchBefore;
  }
  if (setupDone)
  {
    // Check alive-counter
    if (!(readRegisterData[5] == NumModulesFound)) //Module count
    {
      errorByte |= mismatchAfter;
    }
  }

  // Print data received
  if (DebugStatus & DEBUG_TX)
  {
    for (int i = 0; i < 6; i++)
    {
      Serial.print(readRegisterData[i], HEX);
      Serial.print(" ");
    }
    Serial.println();
  }
  if (errorByte) // Error
  {
    if (DebugStatus & DEBUG_TX)
    {
      Serial.println(errorByte, HEX);
      Serial.println("errorByte cleared");
    }
    errorByte &= 0x00; // Clear errors
    resendCounter++;
    if (resendCounter > 2)
    {
      if (DebugStatus & DEBUG_TX)
        Serial.println("WRITEALL fail");
      resendCounter = 0;
      failCounter++;
      return;
    }

    delay(1);
    writeAllSlaves(dataRegister, data, setupDone); // Resend WRITEALL
  }
  else // No errors, clear counters
  {
    resendCounter = 0; // Reset counter
    failCounter = 0; // Reset counter
  }
}
